library IEEE; use IEEE.STD_LOGIC_1164.ALL, IEEE.NUMERIC_STD.ALL;
library UNISIM; use UNISIM.VComponents.all;

entity ClockTree is
    port ( Clk_125MHz : in std_logic; Clk_75MHz, Clk_12_288MHz, Clk_3_072MHz, Clk_500kHz, Clk_250kHz, Clk_48kHz : out std_logic );
end ClockTree;

architecture Archi of ClockTree is
    signal FB1, FB2, ClkU_75MHz, ClkU_48MHz, ClkU_12_288MHz, ClkU_10MHz, ClkU_3_072MHz, ClkU_48kHz : std_logic;
    signal Counter1 : unsigned(3 downto 0);   -- divide by 10 to get 1 MHz
    signal Counter2 : unsigned(1 downto 0);   -- 0: 500k, 1: 250k
    signal Counter3 : unsigned(7 downto 0);   -- 0: 6.144M, 1: 3.072M, 2: 1.536M, 3: 768k, 4: 384k, 5: 192k, 6: 96k, 7: 48k
begin
    MMCME2_BASE_1: MMCME2_BASE
    generic map ( CLKFBOUT_MULT_F => 6.0,       -- Multiply value for all CLKOUT, (2-64), VCO: 750 MHz
                  CLKIN1_PERIOD => 8.0,         -- Input clock period 8ns (125 MHz).
                  CLKOUT0_DIVIDE_F => 15.625, CLKOUT1_DIVIDE => 75, CLKOUT2_DIVIDE => 10, DIVCLK_DIVIDE => 1 )
    port map ( CLKIN1 => Clk_125MHz, CLKFBOUT => FB1, CLKFBIN => FB1, PWRDWN => '0', RST => '0',
               CLKOUT0 => ClkU_48MHz, CLKOUT1 => ClkU_10MHz, CLKOUT2 => ClkU_75MHz );

    MMCME2_BASE_2: MMCME2_BASE
    generic map ( CLKFBOUT_MULT_F => 16.0,      -- Multiply value for all CLKOUT, (2-64), VCO: 768 MHz
                  CLKIN1_PERIOD => 20.833,      -- Input clock period 20.833ns (48 MHz).
                  CLKOUT0_DIVIDE_F => 62.5, DIVCLK_DIVIDE => 1 )
    port map ( CLKIN1 => ClkU_48MHz, CLKFBOUT => FB2, CLKFBIN => FB2, PWRDWN => '0', RST => '0', CLKOUT0 => ClkU_12_288MHz );

    process begin
      wait until rising_edge(ClkU_10MHz);
      if Counter1 = 4d"9" then                  -- generate 1 MHz
        Counter1 <= 4d"0";
        Counter2 <= Counter2 + 1;
      else
        Counter1 <= Counter1 + 1;
      end if;
    end process;

    process begin
      wait until rising_edge(Clk_12_288MHz);
      Counter3 <= Counter3 + 1;
      ClkU_3_072MHz <= not std_logic(Counter3(1));
      ClkU_48kHz <= std_logic(Counter3(7));
    end process;

    BUFG_1: BUFG port map ( Clk_75MHz, ClkU_75MHz );
    BUFG_2: BUFG port map ( Clk_12_288MHz, ClkU_12_288MHz );
    BUFG_3: BUFG port map ( Clk_3_072MHz, ClkU_3_072MHz );
    BUFG_4: BUFG port map ( Clk_500kHz, Counter2(0) );
    BUFG_5: BUFG port map ( Clk_250kHz, Counter2(1) );
    BUFG_6: BUFG port map ( Clk_48kHz, ClkU_48kHz );
end Archi;