library IEEE; use IEEE.STD_LOGIC_1164.ALL, IEEE.NUMERIC_STD.ALL;
library UNISIM; use UNISIM.VComponents.all;

entity i2c_out is
 Port   ( 
        Clk : in std_logic;
        SDA : out std_logic;
        SCL : out std_logic;
		bitCounter : out unsigned(4 downto 0);               -- 0 bis 18         0,1 = x    2,3 = CH,  4-18=Data
		CHcounter : out unsigned(1 downto 0)                -- 

        );
end i2c_out;

architecture Behavioral of i2c_out is
    type ROM is array (0 to 26) of std_logic_vector(7 downto 0);                                                 

-- i2c Rom: 
--        8E = Start
--        FF = Idle
--        06 = 0 oder Ack
--        F6 = 1
--        17 = Stop
signal UM_INIT : ROM := (   X"FF", X"FF", X"FF", X"FF", X"17", X"FF", X"FF", X"FF", X"8E",                         -- Initialisierung
                            X"06", X"F6", X"06", X"F6", X"06", X"06", X"06",        X"F6",    	X"F6",                -- Adressierung     rom_id = 10 - 18
                            -- Adresse                                              -R        	- Ack von ADC
                            X"F6", X"F6", X"F6", X"F6", X"F6", X"F6", X"F6",X"F6", 			X"06",                -- ch 1 					rom_id = 18 - 26  bits: 0-8
							--  x      x     0       1                                                                       rom_id = 20,21 CH_ID
                            -- Daten von ADC ChI   Bit 11-8                                 - Ack von Master                 rom_id = 22-26 D11-D9          bits: 9-17
                            others => X"00");

signal counterB : integer := 3;
signal counterR : integer := 0;

signal frameCounter : integer := 0;             -- 0 bis 1

begin

    
    process begin
        wait until rising_edge(Clk);

  
        if counterB = 3 then
            SCL <= UM_INIT(counterR)(counterB);
            SDA <= UM_INIT(counterR)(counterB+4);
            counterB <= 2;

        elsif counterB = 2 then
            SCL <= UM_INIT(counterR)(counterB);
            SDA <= UM_INIT(counterR)(counterB+4);
            counterB <= 1;
            
        elsif counterB = 1 then
            SCL <= UM_INIT(counterR)(counterB);
            SDA <= UM_INIT(counterR)(counterB+4);
            counterB <= 0;

        elsif counterB = 0 then
            SCL <= UM_INIT(counterR)(counterB);
            SDA <= UM_INIT(counterR)(counterB+4);
            
			
			
            if counterR /= 26 then                  -- wenn Counter kleiner 26 dann weiter
                counterB <= 3;
                counterR <= counterR + 1;
            else                                    
                if bitCounter /= 5d"18" then
					bitCounter <= bitCounter + 1;
                    counterB <= 3;
                    counterR <= 18;
                    if frameCounter = 0 then
                        frameCounter <= 1;
                    else
                        frameCounter <= 0;
                        if CHcounter /= 2d"3" then
                            CHcounter <= CHcounter + 2d"1";
                        else
                            CHcounter <= 2d"0";
                        end if;
                    end if;
				else
					bitCounter <= 5d"0";
				end if;
            end if;
        end if;
    end process;                         
end Behavioral;